<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
	<title>Cahoots - Text In, Structure Out</title>

	<link href='http://fonts.googleapis.com/css?family=Oswald:400,700' rel='stylesheet' type='text/css'>
	<link type="text/css" rel="stylesheet" href="/static/cahoots.css"/>

    <link rel="shortcut icon" href="/static/favicon.ico?src=freepik-flaticon" type="image/x-icon" />


	<script src="//ajax.googleapis.com/ajax/libs/jquery/1.8.0/jquery.min.js"></script>
	<script type="text/javascript" src="/static/cahoots.js"></script>
</head>

<body>

	<div id="header" class="container">
		<div class="container-content">
			<h1><a href="/">Cahoots</a></h1>
			<h2>Text In &#8658; Structure Out</h2>
		</div>
        <div id="primary" class="bordered container">
            <div class="container-content">
                ${self.primary()}
            </div>
        </div>
	</div>

	<div id="secondary" class="container">
		<div class="container-content">
			${self.secondary()}
		</div>
	</div>

	<div id="footer" class="container">
		<div class="container-content">
			${self.footer()}
		</div>
	</div>

</body>
</html>